%
%  XT_Filaments_Join.m
%
%  Version 1.0  8/6/2017
%  Serge Garbay serge.garbay@inserm.fr
%
%  Run on Imaris x64 7.7.2, Matlab R2009a
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory : XT/matlab
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Filaments Functions">
%        <Item name="Join filaments into one filament." icon="Matlab">
%          <Command>MatlabXT::XT_Filaments_Join(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Join filaments into one filament." icon="Matlab">
%            <Command>MatlabXT::XT_Filaments_Join(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%
%   Join filaments object into one filament (or dendrite)
%
%
function XT_Filaments_Join(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% get the filament
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);

% did we find the filament?
if isequal(vFilaments, [])
    msgbox('XT_Filaments_Join: a Filament object in Surpass Scene is required!');
    return;
end

vCount = vFilaments.GetNumberOfFilaments;
if vCount <= 1
	msgbox(sprintf('XT_Filaments_Join: Got %d filament. Need at least 2 filaments !', vCount));
    return;
end

% Init vFilamentsXYZ with the first dendrite (one XYZ list)
vFilamentsXYZ = vFilaments.GetPositionsXYZ(0);

% Scan filaments (disconected dendrite) from the second to the end
for vFilamentIndex = 1:vCount-1

  % vertical concatenation of the current filament (Dendrite : list of XYZ position)
  aFilamentsXYZ = vFilaments.GetPositionsXYZ(vFilamentIndex);
  vFilamentsXYZ = [vFilamentsXYZ ; aFilamentsXYZ];
  
end

% Create a new connected filament
vNewFilament = vImarisApplication.GetFactory.CreateFilaments;

% vNewFilament.SetName('new filament');
new_filament_name = sprintf('Joined %s', char(vFilaments.GetName));
vNewFilament.SetName(new_filament_name);
    
% set colors
vRGBA = vFilaments.GetColorRGBA;
vNewFilament.SetColorRGBA(vRGBA);

% Required setting !!!
vRed = 255; vGreen = 0; vBlue = 0; vAlpha = 125;
vNewFilament.SetColorSpinesRGBA(vRed + vGreen*256 + vBlue*256^2 + vAlpha*256^3);

% count the vertex number of the resulting filament
nbXYZ = length(vFilamentsXYZ);

% vFilamentsEdges = vFilaments.GetEdges(vFilamentIndex);
% edge structure
vFilamentsEdges = [0:nbXYZ-2;1:nbXYZ-1]';

% vFilamentsRadius = vFilaments.GetRadii(vFilamentIndex);  
% radius structure obtained with the first filament radius
radius_value = vFilaments.GetRadii(0); 
vFilamentsRadius = repmat(radius_value, nbXYZ, 1);

% vFilamentsTypes = vFilaments.GetTypes(vFilamentIndex);
% dendrite structure
vFilamentsTypes = zeros(nbXYZ, 1);

vFilamentsTime = vFilaments.GetTimeIndex(vFilamentIndex);  

% vGetEdgesSegmentId = vFilaments.GetEdgesSegmentId(vFilamentIndex);

% info
% X = ['New XYZ Filament'];      disp(X); disp(vFilamentsXYZ);       disp(size(vFilamentsXYZ));
% X = ['vFilamentsRadius '];     disp(X); disp(vFilamentsRadius);    disp(size(vFilamentsRadius));
% X = ['vFilamentsTypes '];      disp(X); disp(vFilamentsTypes);     disp(size(vFilamentsTypes));
% X = ['vFilamentsEdges '];      disp(X); disp(vFilamentsEdges);
% X = ['vFilamentsTime '];       disp(X); disp(vFilamentsTime);

vNewFilament.AddFilament(vFilamentsXYZ, vFilamentsRadius, vFilamentsTypes, vFilamentsEdges, vFilamentsTime);

vImarisApplication.GetSurpassScene.AddChild(vNewFilament, -1);

% msgbox(sprintf('XT_Filaments_Join: Ok, %s created.', new_filament_name));
disp(sprintf('XT_Filaments_Join: Ok, %s created.', new_filament_name));

% just to see message on the Matlab window
pause(2);
